/* 3_senzavocali.c */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* FUNZIONE DA REALIZZARE
   ======================

   Realizzare la funzione senzavocali(s) riportata sotto, che riceve
   in input una stringa s e restituisce una NUOVA stringa contenente
   gli stessi caratteri di s tranne le vocali. 

   Ad esempio, se s = "questa e' una prova.", la funzione deve
   restituire la nuova stringa "qst ' n prv.".

   Si garantisce che s non contenga lettere maiuscole, ma potra'
   contenere spazi, numeri e altri caratteri di punteggiatura (che non
   sono vocali, quindi vanno riportati nella stringa risultato).  La
   funzione NON DEVE modificare la stringa passata come parametro.

   Se lo si ritiene utile e' possibile usare funzioni definite in
   <string.h> come ad esempio strlen().

*/

/* Funzione di supporto che ritorna 1 se e solo se c e' una vocale
   (minuscola) */
int vocale(char c)
{
    return ('a' == c || 'e' == c || 'i' == c || 'o' == c || 'u' == c);
}

char *senzavocali( char *s )
{
    char *r = NULL;
    int i, j, n = 0; /* lunghezza del risultato */
    /* prima fase: contiamo quanti sono i caratteri non-vocali, in
       modo da sapere quanto dovra' essere lunga la stringa
       risultato */
    for (i=0; s[i]; i++) {
        if (!vocale(s[i])) n++;
    }
    /* Allochiamo lo spazio strettamente necessario al risultato; si
       noti che dobbiamo allocare un byte in piu' per il
       terminatore. Moltiplicare per sizeof(char) non e' strettamente
       necessario, dato che il linguaggio garantisce che sizeof(char)
       == 1. */
    r = (char*)malloc((n+1)*sizeof(char));
    /* seconda fase: copiamo i caratteri non-vocali dalla stringa s
       alla nuova stringa r */
    for (i=j=0; s[i]; i++) {
        if (!vocale(s[i])) {
            r[j] = s[i];
            j++;
        }
    }
    /* ricordiamoci di terminare correttamente la stringa r */
    r[j] = '\0';
    return r;
}

void test(char *s, char *expect)
{
    char *r = senzavocali(s);
    if ( (r != NULL) && (0 == strcmp(r, expect)) ) {
        printf("Test OK\n");
    } else {
        printf("Test FALLITO: s=\"%s\", risultato atteso \"%s\", risultato ottenuto \"%s\"\n", s, expect, r);
    }
    free(r);
}

int main( void )
{
    test("questa e' una prova.", "qst ' n prv.");
    test("supercalifragilistichespiralidoso", "sprclfrglstchsprlds");
    test("aaa", "");
    test("una mela al giorno toglie il medico di torno", "n ml l grn tgl l mdc d trn");
    test("alcuni studi dimostrano che e' spesso possibile comprendere un testo anche senza vocali", "lcn std dmstrn ch ' spss pssbl cmprndr n tst nch snz vcl");
    test("", "");

    return 0;    
}
